/* X-Chat
 * Copyright (C) 1998 Peter Zelezny.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "../../config.h"
#ifdef HAVE_STRINGS_H
#include <strings.h>
#endif
#include <unistd.h>
#include "../common/xchat.h"
#include "simple.h"
#include "gtkutil.h"
#include <fcntl.h>
#include <time.h>
#include <gdk/gdkkeysyms.h>

extern int tcp_send (struct server *serv, char *buf);
extern struct server *new_server (void);
extern struct session *new_session (struct server *serv, char *name);
extern void connect_server (struct session *sess, 
			    char *server, 
			    int port, 
			    int quiet);

void
fe_add_chan_list (struct server *serv, char *chan, char *users, char *topic)
{
}

struct ChannelData {
	ChannelInfo *info;

	server *serv;
	GtkWidget *join;
};

static void
chanlist_join_clicked (GtkWidget * wid,
		       struct ChannelData *data)
{
	ChannelInfo *info = data->info;

	g_return_if_fail (info != NULL);

	if (info->server &&
	    g_strcasecmp (info->server, info->serv->hostname) != 0) {
		struct server *new_serv = new_server ();
		struct session *new_sess = new_session (new_serv, NULL);
		new_sess->server->front_session = new_sess;
		strcpy (new_sess->willjoinchannel, info->channel);
		connect_server (new_sess,
				info->server,
				info->port,
				FALSE);
	} else if (info->serv->connected) {
		gchar tbuf [256];
		snprintf (tbuf, sizeof (tbuf), "JOIN %s\r\n", info->channel);
		tcp_send (info->serv, tbuf);
	}
	
	gtk_widget_destroy (info->serv->gui->chanlist_window);
}

static void 
chanlist_closegui (GtkWidget * wid, 
		   struct ChannelData *data)
{
	struct server *serv = data->serv;
	
	serv->gui->chanlist_window = 0;
}

static void
select_child_cb (GtkList *list,
		 GtkWidget *child,
		 struct ChannelData *data)
{
	data->info = gtk_object_get_data (GTK_OBJECT (child), "channel-info");
	gtk_widget_set_sensitive (data->join, TRUE);
}

static void
unselect_child_cb (GtkList *list,
		   GtkWidget *child,
		   struct ChannelData *data)
{
	data->info = NULL;
	gtk_widget_set_sensitive (data->join, FALSE);
}

GtkWidget *
chan_list_window_new (struct server *serv)
{
	GtkWidget *chan_list_window;
	GtkWidget *dialog_vbox;
	GtkWidget *scrolledwindow1;
	GtkWidget *viewport1;
	GtkWidget *label1;
	GtkWidget *label2;
	GtkWidget *hseparator1;
	GtkWidget *dialog_action_area1;
	GtkWidget *button2;
	GtkWidget *button3;
	GtkWidget *chan_list;
	GSList *chan_iter = serv->gui->channel_info;
	GList *channels = NULL;
	struct ChannelData *data;

	data = g_new (struct ChannelData, 1);
	data->serv = serv;

	chan_list_window = gnome_dialog_new (_("Support Channel Listing"), 
					     NULL);
	gtk_signal_connect (GTK_OBJECT (chan_list_window), "destroy",
			    GTK_SIGNAL_FUNC (chanlist_closegui), data);

	gtk_window_set_policy (GTK_WINDOW (chan_list_window), 
			       TRUE, 
			       TRUE, 
			       FALSE);
	gtk_window_set_default_size (GTK_WINDOW (chan_list_window), 
				     300,
				     350);
	gnome_dialog_set_close (GNOME_DIALOG (chan_list_window), TRUE);

	dialog_vbox = GNOME_DIALOG (chan_list_window)->vbox;
	gtk_widget_show (dialog_vbox);

	scrolledwindow1 = gtk_scrolled_window_new (NULL, NULL);
	gtk_widget_show (scrolledwindow1);
	gtk_box_pack_start (GTK_BOX (dialog_vbox), 
			    scrolledwindow1, 
			    TRUE, 
			    TRUE, 
			    0);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow1), 
					GTK_POLICY_NEVER, 
					GTK_POLICY_AUTOMATIC);

	viewport1 = gtk_viewport_new (NULL, NULL);
	gtk_widget_show (viewport1);
	gtk_container_add (GTK_CONTAINER (scrolledwindow1), viewport1);

	chan_list = gtk_list_new ();
	gtk_signal_connect (GTK_OBJECT (chan_list), "select_child",
			    GTK_SIGNAL_FUNC (select_child_cb), data);
	gtk_signal_connect (GTK_OBJECT (chan_list), "unselect_child",
			    GTK_SIGNAL_FUNC (unselect_child_cb), data);
	gtk_widget_show (chan_list);
	gtk_container_add (GTK_CONTAINER (viewport1), chan_list);

	while (chan_iter) {
		GtkWidget *item, *vbox;
		ChannelInfo *info = chan_iter->data;
		gchar *pattern;

		item = gtk_list_item_new ();
		gtk_container_set_border_width (GTK_CONTAINER (item), 2);

		vbox = gtk_vbox_new (FALSE, 8);
		gtk_container_add (GTK_CONTAINER (item), vbox);

		label1 = gtk_label_new (info->name);
		gtk_widget_show (label1);
		gtk_box_pack_start (GTK_BOX (vbox), label1, TRUE, TRUE, 0);
		gtk_label_set_justify (GTK_LABEL (label1), GTK_JUSTIFY_LEFT);
		gtk_misc_set_alignment (GTK_MISC (label1), 0.0, 0.0);

		pattern = g_strnfill (strlen (info->name), '_');
		gtk_label_set_pattern (GTK_LABEL (label1), pattern);
		g_free (pattern);

		label2 = gtk_label_new (info->desc);
		gtk_widget_show (label2);
		gtk_box_pack_start (GTK_BOX (vbox), label2, TRUE, TRUE, 0);
		gtk_label_set_justify (GTK_LABEL (label2), GTK_JUSTIFY_LEFT);
		gtk_label_set_line_wrap (GTK_LABEL (label2), TRUE);
		gtk_misc_set_alignment (GTK_MISC (label2), 0.0, 0.0);

		gtk_widget_show_all (item);
		gtk_object_set_data (GTK_OBJECT (item), "channel-info", info);

		channels = g_list_prepend (channels, item);

		if (chan_iter->next) {
			hseparator1 = gtk_hseparator_new ();
			gtk_widget_show (hseparator1);

			gtk_box_pack_start (GTK_BOX (vbox), 
					    hseparator1, 
					    TRUE, 
					    TRUE, 
					    0);
		}

		chan_iter = chan_iter->next;
	}

	channels = g_list_reverse (channels);
	gtk_list_append_items (GTK_LIST (chan_list), channels);

	dialog_action_area1 = GNOME_DIALOG (chan_list_window)->action_area;
	gtk_widget_show (dialog_action_area1);
	gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area1), 
				   GTK_BUTTONBOX_SPREAD);
	gtk_button_box_set_spacing (GTK_BUTTON_BOX (dialog_action_area1), 8);

	gnome_dialog_append_button_with_pixmap (GNOME_DIALOG (chan_list_window),
						_("Join Channel"), 
						GNOME_STOCK_PIXMAP_JUMP_TO);
	button2 = g_list_last (GNOME_DIALOG (chan_list_window)->buttons)->data;
	data->join = button2;
	gtk_widget_set_sensitive (data->join, FALSE);

	gtk_signal_connect (GTK_OBJECT (button2), "clicked",
			    GTK_SIGNAL_FUNC (chanlist_join_clicked),
			    data);	

	gtk_widget_show (button2);
	GTK_WIDGET_SET_FLAGS (button2, GTK_CAN_DEFAULT);

	gnome_dialog_append_button_with_pixmap (GNOME_DIALOG (chan_list_window),
						_("Close"), 
						GNOME_STOCK_PIXMAP_CLOSE);
	button3 = g_list_last (GNOME_DIALOG (chan_list_window)->buttons)->data;

	gtk_widget_show (button3);
	GTK_WIDGET_SET_FLAGS (button3, GTK_CAN_DEFAULT);

	return chan_list_window;
}

void chanlist_opengui (struct server *serv)
{
	GtkWidget *win;

	if (serv->gui->chanlist_window) {
		gdk_window_show (serv->gui->chanlist_window->window);
		return;
	}

	if (!serv->gui->channel_info) {
		gtkutil_simpledialog ("Unable to fetch Channel List");
		return;
	}

	win = chan_list_window_new (serv);

	gtk_widget_show (win);

	serv->gui->chanlist_window = win;

	return;
}
