/* X-Chat
 * Copyright (C) 1998 Peter Zelezny.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <string.h>

#include "../common/fe.h"
#include "../common/util.h"
#include "../common/xchat.h"
#include "gtkutil.h"
#include "menu.h"
#include "simple.h"
#include "xtext.h"

extern void xchat_exec (char *cmd);
extern int handle_command (char *cmd, struct session *sess, int, int);
extern int auto_insert (char *dest, 
			char *src, 
			char *word[], 
			char *word_eol[],
			char *a, 
			char *c, 
			char *d, 
			char *h, 
			char *n, 
			char *s);
extern int cfg_get_bool (char *var);
GtkWidget *createmenus (void *app, struct session * sess, int bar);
extern int tcp_send (struct server *serv, char *buf);
extern void search_open (session * sess);

extern struct session *menu_sess;
extern GSList *sess_list;

extern GSList *popup_list;
extern GSList *button_list;
extern GSList *command_list;
extern GSList *ctcp_list;
extern GSList *replace_list;
extern GSList *usermenu_list;
extern GSList *urlhandler_list;
static GSList *submenu_list;

void goto_url (char *url)
{
	gnome_url_show (url);
}

/* execute a userlistbutton/popupmenu command */

static void nick_command (session * sess, char *cmd)
{
	if (*cmd == '!')
		xchat_exec (cmd + 1);
	else
		handle_command (cmd, sess, FALSE, FALSE);
}

/* fill in the %a %s %n etc and execute the command */

void nick_command_parse (session * sess, char *cmd, char *nick, char *allnick)
{
	char *buf;
	char *host;
	struct User *user;

	user = find_name (sess, nick);
	if (user && user->hostname)
		host = strchr (user->hostname, '@') + 1;
	else
		host = "Host unknown";

	/* this can't overflow, since popup->cmd is only 256 */
	buf = malloc (strlen (cmd) + strlen (nick) + strlen (allnick) + 512);

	auto_insert (buf, cmd, 0, 0, allnick, sess->channel, "", host,
		     sess->server->nick, nick);

	nick_command (sess, buf);

	free (buf);
}

/* userlist button has been clicked */

void userlist_button_cb (GtkWidget * button, char *cmd)
{
	int nicks, using_allnicks = FALSE;
	char *nick = NULL, *allnicks;
	struct User *user;
	GList *sel_list;
	session *sess;

	/* this is set in maingui.c userlist_button() */
	sess = gtk_object_get_user_data (GTK_OBJECT (button));

	if (strstr (cmd, "%a"))
		using_allnicks = TRUE;

	/* count number of selected rows */
	sel_list = (GTK_CLIST (sess->gui->namelistgad))->selection;
	nicks = g_list_length (sel_list);

	if (nicks == 0) {
		nick_command_parse (sess, cmd, "", "");
		return;
	}

	/* create "allnicks" string */
	allnicks = malloc (65 * nicks);
	*allnicks = 0;

	nicks = 0;
	while (sel_list) {
		user =
		    gtk_clist_get_row_data (GTK_CLIST (sess->gui->namelistgad),
					    (gint) sel_list->data);
		if (!nick)
			nick = user->nick;
		if (nicks > 0)
			strcat (allnicks, " ");
		strcat (allnicks, user->nick);
		sel_list = sel_list->next;
		nicks++;

		/* if not using "%a", execute the command once for each
                   nickname. */
		if (!using_allnicks)
			nick_command_parse (sess, cmd, user->nick, "");

	}

	if (using_allnicks) {
		if (!nick)
			nick = "";
		nick_command_parse (sess, cmd, nick, allnicks);
	}

	free (allnicks);
}

/* a popup-menu-item has been selected */

static void popup_menu_cb (GtkWidget * item, char *cmd)
{
	char *nick;

	/* the userdata is set in menu_quick_item() */
	nick = gtk_object_get_user_data (GTK_OBJECT (item));

	if (!menu_sess)		/* for url grabber window */
		nick_command_parse (sess_list->data, cmd, nick, nick);
	else
		nick_command_parse (menu_sess, cmd, nick, nick);
}

static void
menu_toggle_item (char *label, GtkWidget * menu, void *callback, void *userdata,
		  int state)
{
	GtkWidget *item;

	item = gtk_check_menu_item_new_with_label (label);
	/* Always show checkbox, not just when it is moused over. */
	gtk_check_menu_item_set_show_toggle ((GtkCheckMenuItem *) item, TRUE);
	gtk_check_menu_item_set_state ((GtkCheckMenuItem *) item, state);
	gtk_menu_append (GTK_MENU (menu), item);
	gtk_signal_connect (GTK_OBJECT (item), "activate",
			    GTK_SIGNAL_FUNC (callback), userdata);
	gtk_widget_show (item);
}

static GtkWidget *menu_quick_item (char *cmd, char *label, GtkWidget * menu,
				   int flags, gpointer userdata)
{
	GtkWidget *item;
	if (!label)
		item = gtk_menu_item_new ();
	else
		item = gtk_menu_item_new_with_label (label);
	gtk_menu_append (GTK_MENU (menu), item);
	gtk_object_set_user_data (GTK_OBJECT (item), userdata);
	if (cmd)
		gtk_signal_connect (GTK_OBJECT (item), "activate",
				    GTK_SIGNAL_FUNC (popup_menu_cb), cmd);
	if (flags & (1 << 0))
		gtk_widget_set_sensitive (GTK_WIDGET (item), FALSE);
	gtk_widget_show (item);

	return item;
}

void
menu_quick_item_with_callback (void *callback, char *label, GtkWidget * menu,
			       void *arg)
{
	GtkWidget *item;

	item = gtk_menu_item_new_with_label (label);
	gtk_menu_append (GTK_MENU (menu), item);
	gtk_signal_connect (GTK_OBJECT (item), "activate",
			    GTK_SIGNAL_FUNC (callback), arg);
	gtk_widget_show (item);
}

static GtkWidget *menu_quick_sub (char *name, GtkWidget * menu)
{
	GtkWidget *sub_menu;
	GtkWidget *sub_item;

	if (!name)
		return menu;

	/* Code to add a submenu */
	sub_menu = gtk_menu_new ();
	sub_item = gtk_menu_item_new_with_label (name);
	gtk_menu_append (GTK_MENU (menu), sub_item);
	gtk_widget_show (sub_item);
	gtk_menu_item_set_submenu (GTK_MENU_ITEM (sub_item), sub_menu);

	/* We create a new element in the list */
	submenu_list = g_slist_prepend (submenu_list, sub_menu);
	return (sub_menu);
}

static GtkWidget *menu_quick_endsub ()
{
	/* Just delete the first element in the linked list pointed to by first */
	if (submenu_list)
		submenu_list =
		    g_slist_remove (submenu_list, submenu_list->data);

	if (submenu_list)
		return (submenu_list->data);
	else
		return NULL;
}

static void toggle_cb (GtkWidget * item, char *pref_name)
{
	char buf[256];

	if (GTK_CHECK_MENU_ITEM (item)->active)
		snprintf (buf, sizeof (buf), "/set %s 1", pref_name);
	else
		snprintf (buf, sizeof (buf), "/set %s 0", pref_name);

	handle_command (buf, menu_sess, FALSE, FALSE);
}

/* append items to "menu" using the (struct popup*) list provided */

void menu_create (GtkWidget * menu, GSList * list, char *target)
{
	struct popup *pop;
	GtkWidget *tempmenu = menu;

	submenu_list = g_slist_prepend (0, menu);
	while (list) {
		pop = (struct popup *) list->data;
		if (!strncasecmp (pop->name, "SUB", 3))
			tempmenu = menu_quick_sub (pop->cmd, tempmenu);
		else if (!strncasecmp (pop->name, "TOGGLE", 6)) {
			menu_toggle_item (pop->name + 7, tempmenu, toggle_cb,
					  pop->cmd, cfg_get_bool (pop->cmd));
		} else if (!strncasecmp (pop->name, "ENDSUB", 6)) {
			if (tempmenu != menu)
				tempmenu = menu_quick_endsub ();
			/* If we get here and tempmenu equals menu that means we havent got any submenus to exit from */
		} else if (!strncasecmp (pop->name, "SEP", 3))
			menu_quick_item (0, 0, tempmenu, 1, 0);
		else
			menu_quick_item (pop->cmd, pop->name, tempmenu, 0,
					 target);

		list = list->next;

	}

	/* Let's clean up the linked list from mem */
	while (submenu_list)
		submenu_list =
		    g_slist_remove (submenu_list, submenu_list->data);
}

static void menu_popup (GtkWidget * menu, GdkEventButton * event)
{
	gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL,
			event->button, event->time);
	gtk_widget_show (menu);
}

static char *str_copy = 0;	/* for all pop-up menus */

void menu_nickmenu (session * sess, GdkEventButton * event, char *nick)
{
	char buf[256];
	struct User *user;
	GtkWidget *wid = NULL, *submenu = NULL, *menu;// = gtk_menu_new ();
	gchar *user_country;

	//menu = createmenus (0, sess, 0);
	menu = gtk_menu_new ();

	if (!menu_sess) goto NO_USERINFO;

	if (str_copy) free (str_copy);
	str_copy = strdup (nick);

	submenu_list = 0;	/* first time though, might not be 0 */

	user = find_name_global (menu_sess->server, nick);
	if (!user) goto NO_USERINFO;

	if (!user->hostname || !user->realname || !user->lasttalk)
		goto NO_USERINFO;

	submenu = menu_quick_sub (str_copy, menu);

	if (user->realname && strcmp (user->realname, nick) != 0) {
		sprintf (buf, "Name: %s", user->realname);
		wid = menu_quick_item (0, buf, submenu, 0, 0);
	}

	if (user->hostname) {
		sprintf (buf, "User: %s", user->hostname);
		wid = menu_quick_item (0, buf, submenu, 0, 0);
		
		if ((user_country = country (user->hostname))) {
			sprintf (buf, "Country: %s", user_country);
			wid = menu_quick_item (0, buf, submenu, 0, 0);
		}
	}
	
	// signon
	
	// channels joined

	if (user->lasttalk) {
		sprintf (buf, 
			 "Last Msg: %s", 
			 ctime (&(user->lasttalk)));
		
		buf[strlen (buf) - 1] = 0;
		
		wid = menu_quick_item (0, buf, submenu, 0, 0);
	}

	menu_quick_endsub ();
	menu_quick_item (0, 0, menu, 1, 0);

 NO_USERINFO:
	menu_create (menu, popup_list, str_copy);
	menu_popup (menu, event);
}

void menu_middlemenu (session * sess, GdkEventButton * event)
{
	GtkWidget *menu;

	menu = createmenus (0, sess, 0);

	menu_popup (menu, event);
}

void menu_urlmenu (session * sess, GdkEventButton * event, char *url)
{
	GtkWidget *menu;
	int i;
	char *tmp;

	if (str_copy)
		free (str_copy);

	/* replace all commas with %2c */
	str_copy = malloc ((strlen (url) * 3) + 1);
	i = 0;
	while (*url) {
		if (*url == ',') {
			str_copy[i++] = '%';
			str_copy[i++] = '2';
			str_copy[i] = 'c';
		} else
			str_copy[i] = *url;
		i++;
		url++;
	}
	str_copy[i] = 0;

	menu = gtk_menu_new ();
	if (strlen (str_copy) > 51) {
		tmp = strdup (str_copy);
		tmp[47] = tmp[48] = tmp[49] = '.';
		tmp[50] = 0;
		menu_quick_item (0, tmp, menu, 1, 0);
		free (tmp);
	} else {
		menu_quick_item (0, str_copy, menu, 1, 0);
	}
	menu_quick_item (0, 0, menu, 1, 0);

	menu_create (menu, urlhandler_list, str_copy);
	menu_popup (menu, event);
}

static void menu_chan_join (GtkWidget * menu, char *chan)
{
	char tbuf[256];

	if (menu_sess) {
		snprintf (tbuf, sizeof tbuf, "JOIN %s\r\n", chan);
		tcp_send (menu_sess->server, tbuf);
	}
}

void menu_chanmenu (struct session *sess, GdkEventButton * event, char *chan)
{
	GtkWidget *menu;
	GSList *list = sess_list;
	int is_joined = FALSE;
	struct session *s;

	while (list) {
		s = (struct session *) list->data;
		if (s->server == sess->server) {
			if (!strcasecmp (chan, s->channel)) {
				is_joined = TRUE;
				break;
			}
		}
		list = list->next;
	}

	if (is_joined) return;

	if (str_copy)
		free (str_copy);
	str_copy = strdup (chan);

	menu = gtk_menu_new ();

	menu_quick_item (0, chan, menu, 1, str_copy);
	menu_quick_item (0, 0, menu, 1, str_copy);

	menu_quick_item_with_callback (menu_chan_join, 
				       "Join channel in New Window", 
				       menu, 
				       str_copy);

	gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL,
			event->button, event->time);
}

static void menu_close (GtkWidget * wid, gpointer none)
{
	gtk_widget_destroy (menu_sess->gui->window);
}

static void menu_search ()
{
	search_open (menu_sess);
}

static void savebuffer_req_done (session * sess, void *arg2, char *file)
{
	int fh, newlen;
	char *buf;
	textentry *ent;

	if (!file)
		return;

	fh = open (file, O_TRUNC | O_WRONLY | O_CREAT, 0600);
	if (fh != -1) {
		ent = GTK_XTEXT (sess->gui->textgad)->text_first;
		while (ent) {
			buf = gtk_xtext_strip_color (
					ent->str, 
					ent->str_len, 
					NULL,
					&newlen,
					GTK_XTEXT (sess->gui->textgad)->fonttype);
			write (fh, buf, strlen (buf));
			write (fh, "\n", 1);
			free (buf);
			ent = ent->next;
		}
		close (fh);
	}
	free (file);
}

static void menu_savebuffer (GtkWidget * wid, gpointer none)
{
	gtkutil_file_req ("Select an output filename", savebuffer_req_done,
			  menu_sess, 0, TRUE);
}

static GnomeUIInfo mainmenu[] = {
	{
	 GNOME_APP_UI_ITEM,
	 N_("Search..."), 0,
	 menu_search, 0, 0,
	 GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_SEARCH,
	 0, 0, 0},
	{
	 GNOME_APP_UI_ITEM,
	 N_("Save Chat History..."), 0,
	 menu_savebuffer, 0, 0, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_SAVE,
	 0,
	 0, 0},
        {
         GNOME_APP_UI_ITEM,
         N_("Close Window"), 0,
         menu_close, 0, 0, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_QUIT,
         0,
	 0, 0},
	GNOMEUIINFO_END
};

GtkWidget *createmenus (void *app, struct session *sess, int bar)
{
	GtkWidget *menu;

	if (!menu_sess)
		menu_sess = sess;

	if (bar) {
		gnome_app_create_menus (GNOME_APP (app), mainmenu);
		menu = GNOME_APP (app)->menubar;
	} else {
		menu = gnome_popup_menu_new (mainmenu);
	}

	return menu;
}
